/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Le Framework de jeux JavaScript Phaser
 *
Phaser v2.0.5 - http://phaser.io - @photonstorm - (c) 2014 Photon Storm Ltd. 
https://github.com/photonstorm/phaser
http://examples.phaser.io/
 */



/**
 * Les variables du jeu
 * 
 */
var game; 

var nWidth = 640;
var nHeight = 380;


// les variables du jeu
var map;// la carte du jeu

// pour sauvegarder la position de départ lors du "touch" sur l'écran
var startX;
var startY;

var moveIndex;// pour gérer le multitouch
var mapSpeed = 1;// vitesse de déplacement de la carte - plus le chiffre est élevé plus le déplacement est accéléré
var mapGroup;// groupe contenant les positions de la carte et de la ville
var candidateTown;// la ville sélectionnée par le joueur
var oTextInfo;// affichage d'informations pour le joueur



document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page



/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
    console.log("Main");

    // creation de la zone de jeu - API canvas
    game = new Phaser.Game(nWidth, nHeight, Phaser.AUTO, "idGameDiv", {preload:onPreload, create:onCreate, update:onUpdate});

    // les variables globales du jeu
    game.global = {
        thumbRows : 5// nombre d'icone par ligne  game.global.thumbRows
    }

}


/**
 * Préchargement des éléments du jeu
 * 
 * @return {[type]} [description]
 */
function onPreload()
{ 
    // charge les animations / graphismes du jeu
    game.load.image("map", "assets/map-game.png");
    game.load.image("town", "assets/town.png");
    game.stage.backgroundColor = '#cccccc';// la couleur de fond du jeu
}


/**
 * Affiche le jeu en plein écran
 * 
 * @return {[type]} [description]
 */
function goFullScreen()
{
    game.scale.pageAlignHorizontally = true;
    game.scale.pageAlignVertically = true;
    game.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
    game.scale.setScreenSize(true);
}


/**
 * Création de la scène du jeu,
 * mise en place des graphismes...
 * 
 * @return {[type]} [description]
 */
function onCreate() 
{
    mapGroup = game.add.group();

    // ajout de la carte
    map = game.add.image(0, 0, "map");
    mapGroup.add(map);
    
    // positionne aléatoirement 10 villes
    for(var i = 0;i < 10; i++)
    {
       var town = game.add.image(game.rnd.between(50, map.width - 50), game.rnd.between(50, map.height - 50), "town");
       town.anchor.setTo(0.5);

       town.number = i;// ajout d'une propriété à la ville

       // chaque ville est active sur un "touch" de l'écran
       town.inputEnabled = true;

       // ajout des écouteurs sur les villes
       town.events.onInputDown.add(selectTown, this);
       town.events.onInputUp.add(confirmTown, this);
       mapGroup.add(town);
    }

    // centre la carte
    mapGroup.x = (game.width - map.width) / 2;
    mapGroup.y = (game.height - map.height) / 2;

    // écouteur du "touch" de l'écran par le joueur
    game.input.onDown.add(onTouchMap, this);   


    // ajout d'un texte informatif pour le joueur
    var sText = "Sélectionnez une ville pour commencer le jeu.";
    var style = { font: "20px Arial", fill: "#000000", align: "center" };

    oTextInfo = game.add.text(game.world.centerX-300, 0, sText, style); 
}


/**
 * Appelée en continu pendant le jeu
 * 
 * @return {[type]} [description]
 */
function onUpdate() 
{
}


/**
 * Le joueur effectue un "touch" en continu sur la carte
 * pour la déplacer
 * 
 * @return {[type]} [description]
 */
function onTouchMap()
{
    // sauvegarde la position de départ du "touch"
    startX = game.input.worldX;
    startY = game.input.worldY;

    mapGroup.saveX = mapGroup.x;
    mapGroup.saveY = mapGroup.y;

    // mets à jour les écouteurs sur le déplacement de la carte
    game.input.onDown.remove(onTouchMap);
    game.input.onUp.add(stopMap);

    // ajout un écouteur - appel la fonction dragMap
    moveIndex = game.input.addMoveCallback(dragMap, this)   
}


/**
 * Déplacement de la carte en fonction 
 * des mouvements du joueur et de sa vitesse
 * 
 * @return {[type]} [description]
 */
function dragMap()
{
    // récupération de la position de la souris
    var currentX = game.input.worldX;
    var currentY = game.input.worldY;

    // calcul du déplacement par rapport à la position de départ
    var deltaX = startX - currentX;
    var deltaY = startY - currentY; 

    // si la  carte est en cours de déplacement, une ville ne peut pas être sélectionnée
    if(deltaX * deltaX + deltaY * deltaY > 25)
        candidateTown = null;
    
    // mise à jour de la position de la carte
    mapGroup.x = mapGroup.saveX - deltaX * mapSpeed;
    mapGroup.y = mapGroup.saveY - deltaY * mapSpeed;

    // Permet de limiter le déplacement 
    // en fonction de la taille de la div du jeu et de la carte
    if(mapGroup.x < - map.width + game.width)
        mapGroup.x = - map.width + game.width;
      
    if(mapGroup.x > 0)
        mapGroup.x = 0;
      
    if(mapGroup.y < - map.height + game.height)
        mapGroup.y = - map.height + game.height;
      
    if(mapGroup.y > 0)
        mapGroup.y = 0;
      
}


/**
 * Le joueur relâche le "touch"
 * et arrête de déplacement la carte
 * 
 * @return {[type]} [description]
 */
function stopMap()
{
    // mise à jour des écouteurs
    game.input.onDown.add(onTouchMap);
    game.input.onUp.remove(stopMap);

    game.input.deleteMoveCallback(moveIndex);
}
 

/**
 * Le joueur a sélectionné une ville
 * 
 * @param  {[type]} sprite  [description]
 * @param  {[type]} pointer [description]
 * @return {[type]}         [description]
 */
function selectTown(sprite, pointer)
{
    candidateTown = sprite;// récupère l'objet ville sélectionnée
}
 

/**
 * Le joueur arrête de toucher une ville
 * il confirme sa sélection
 * 
 * @param  {[type]} sprite  [description]
 * @param  {[type]} pointer [description]
 * @return {[type]}         [description]
 */
function confirmTown(sprite, pointer) 
{
    // ajoute un affichage texte de la ville sélectionnée
    if(candidateTown == sprite)
    {
        oTextInfo.setText("Vous avez sélectionné la ville "+sprite.number+".");
    }
}


